/*	SPKMeans.cc 

	Implementation of the SPKMeans class
*/

#include <time.h>
#include <assert.h>
#include <iostream.h>
#include <stdio.h>
#include <fstream.h>
#include <string.h>
#include "mat_vec.h"
#include "SPKMeans.h"

extern long memory_consume;

//#define CV_Sim(i, j) CV_Sim[(i>=j)?(i*(i+1)/2+j):(j*(j+1)/2+i)]

SPKMeans::SPKMeans(SparseMatrixDouble *p_Docs, int cluster[], int num_clusters,
		int init_method, float epsilon)
{
	assert(p_Docs != NULL && cluster != NULL && num_clusters > 0 &&  epsilon > 0);

	//pDocs = p_docs;
	Cluster = cluster;
	n_Clusters = num_clusters;
	Init_Method = init_method;
	Epsilon = epsilon;

	n_Words = p_Docs->GetNumRow();
	n_Docs = p_Docs->GetNumCol();
	n_Iters = 0;
	Result = 0;
	Alg = SPHERICAL_K_MEANS;
	Perturb = DEFAULT_PERTURB;
	n_Samples = DEFAULT_NUM_SAMPLE;
	SampleSize = DEFAULT_SAMPLE_SIZE;
	EncodeScheme = NORM_TERM_FREQ;
	ObjFunc = NON_WEIGHTED_OBJ;
	DumpCSize = false;

	Inner_Sim = new float[n_Docs];
	//CV_Sim = new float[n_Clusters * (n_Clusters+1) / 2];
	Sim_Mat = new (float *)[n_Clusters];
	for (int j = 0; j < n_Clusters; j++)
		Sim_Mat[j] = new float[n_Docs];
	memory_consume+=(n_Docs+n_Clusters*n_Docs)*sizeof(float);

	if (init_method != CONCEPT_VECTORS_INIT)
	{
		Concept_Vector = new VECTOR_double[n_Clusters];
		for (int i = 0; i < n_Clusters; i++)
			Concept_Vector[i] = new float[n_Words];
	}

	memory_consume+=n_Clusters*n_Words*sizeof(float)*2;

	old_CV = new VECTOR_double[n_Clusters];
	for (int i = 0; i < n_Clusters; i++)
		old_CV[i] = new float[n_Words];
	diff = new float[n_Clusters];
	ClusterSize = new  int[n_Clusters];
	DocNormSqr = new float[n_Docs];
	CVNormSqr = new float[n_Clusters];

	memory_consume+=(n_Clusters*2+n_Docs)*sizeof(float);
	memory_consume+=n_Clusters+sizeof(int);

	//rand_gen.Set((unsigned)0);

	rand_gen.Set((unsigned)time(NULL));
}

SPKMeans::SPKMeans(DenseMatrixDouble *Dp_Docs, int cluster[], int num_clusters,
		int init_method, float epsilon)
{
  assert(Dp_Docs != NULL && cluster != NULL && num_clusters > 0 &&  epsilon > 0);

  //DpDocs = p_docs;
	Cluster = cluster;
	n_Clusters = num_clusters;
	Init_Method = init_method;
	Epsilon = epsilon;

	n_Words = Dp_Docs->GetNumRow();
	n_Docs = Dp_Docs->GetNumCol();
	n_Iters = 0;
	Result = 0;

	Inner_Sim = new float[n_Docs];
	//	CV_Sim = new float[n_Clusters * (n_Clusters+1) / 2];
	Sim_Mat = new (float *)[n_Clusters];
	for (int j = 0; j < n_Clusters; j++)
		Sim_Mat[j] = new float[n_Docs];

	if (init_method != CONCEPT_VECTORS_INIT)
	{
		Concept_Vector = new VECTOR_double[n_Clusters];
		for (int i = 0; i < n_Clusters; i++)
			Concept_Vector[i] = new float[n_Words];
	}
	old_CV = new VECTOR_double[n_Clusters];
	for (int i = 0; i < n_Clusters; i++)
		old_CV[i] = new float[n_Words];
	diff = new float[n_Clusters];

	ClusterSize = new  int[n_Clusters];
	DocNormSqr = new float[n_Docs];
	CVNormSqr = new float[n_Clusters];

	rand_gen.Set((unsigned)time(NULL));
}

SPKMeans::SPKMeans()
{
	Alg = SPHERICAL_K_MEANS;
	n_Samples = DEFAULT_NUM_SAMPLE;
	Perturb = DEFAULT_PERTURB;
	SampleSize = DEFAULT_SAMPLE_SIZE;
	Init_Method = RANDOM_PERTURB_INIT;
	EncodeScheme = NORM_TERM_FREQ;
	ObjFunc = NON_WEIGHTED_OBJ;
	DumpCSize = false;
	Concept_Vector = NULL;

	Inner_Sim = NULL;
	//CV_Sim = NULL;

	rand_gen.Set((unsigned)time(NULL));
}

void SPKMeans::Init(SparseMatrixDouble *p_Docs, int cluster[], int num_clusters,
		int init_method, float epsilon)
{
	assert(p_Docs != NULL && cluster != NULL && num_clusters > 0 &&  epsilon > 0);

	if (Concept_Vector != NULL && Init_Method != CONCEPT_VECTORS_INIT)
	{
		for (int i = 0; i < n_Clusters; i++)
			delete[] Concept_Vector[i];
		delete[] Concept_Vector;
	}
	//pDocs = p_docs;
	Cluster = cluster;
	n_Clusters = num_clusters;
	Init_Method = init_method;
	Epsilon = epsilon;

	n_Words = p_Docs->GetNumRow();
	n_Docs = p_Docs->GetNumCol();
	n_Iters = 0;
	Result = 0;

	Inner_Sim = new float[n_Docs];
	//CV_Sim = new float[n_Clusters * (n_Clusters+1) / 2];
	Sim_Mat = new (float *)[n_Clusters];
	for (int j = 0; j < n_Clusters; j++)
		Sim_Mat[j] = new float[n_Docs];

	if (init_method != CONCEPT_VECTORS_INIT)
	{
		Concept_Vector = new VECTOR_double[n_Clusters];
		for (int i = 0; i < n_Clusters; i++)
			Concept_Vector[i] = new float[n_Words];
	}
	old_CV = new VECTOR_double[n_Clusters];
	for (int i = 0; i < n_Clusters; i++)
		old_CV[i] = new float[n_Words];
	diff = new float[n_Clusters];

	ClusterSize = new  int[n_Clusters];
	DocNormSqr = new float[n_Docs];
	CVNormSqr = new float[n_Clusters];
}

void SPKMeans::Init(DenseMatrixDouble *Dp_Docs, int cluster[], int num_clusters,
		int init_method, float epsilon)
{
  assert(Dp_Docs != NULL && cluster != NULL && num_clusters > 0 &&  epsilon > 0);

	if (Concept_Vector != NULL && Init_Method != CONCEPT_VECTORS_INIT)
	{
		for (int i = 0; i < n_Clusters; i++)
			delete[] Concept_Vector[i];
		delete[] Concept_Vector;
	}
	//DpDocs = p_docs;
	Cluster = cluster;
	n_Clusters = num_clusters;
	Init_Method = init_method;
	Epsilon = epsilon;

	n_Words = Dp_Docs->GetNumRow();
	n_Docs = Dp_Docs->GetNumCol();
	n_Iters = 0;
	Result = 0;

	Inner_Sim = new float[n_Docs];
	//	CV_Sim = new float[n_Clusters * (n_Clusters+1) / 2];
	Sim_Mat = new (float *)[n_Clusters];
	for (int j = 0; j < n_Clusters; j++)
		Sim_Mat[j] = new float[n_Docs];

	if (init_method != CONCEPT_VECTORS_INIT)
	{
		Concept_Vector = new VECTOR_double[n_Clusters];
		for (int i = 0; i < n_Clusters; i++)
			Concept_Vector[i] = new float[n_Words];
	}
	old_CV = new VECTOR_double[n_Clusters];
	for (int i = 0; i < n_Clusters; i++)
		old_CV[i] = new float[n_Words];
	diff = new float[n_Clusters];

	ClusterSize = new  int[n_Clusters];
	DocNormSqr = new float[n_Docs];
	CVNormSqr = new float[n_Clusters];
}

SPKMeans::~SPKMeans()
{
	if (Concept_Vector != NULL && Init_Method != CONCEPT_VECTORS_INIT)
	{
		for (int i = 0; i < n_Clusters; i++)
		  {
		    delete[] Concept_Vector[i];
		    delete[] Sim_Mat[i];
		  }
		delete[] Sim_Mat;
		delete[] Concept_Vector;
	}
	delete[] ClusterSize; // any problem? ...
	delete[] DocNormSqr;
	delete[] CVNormSqr;
	delete[] Inner_Sim;
	//delete[] CV_Sim;
}

void SPKMeans::ReturnConVec(VECTOR_double * ConVec)
{
  for ( int i=0; i<n_Clusters; i++)
      for ( int j=0; j<n_Words; j++)
	ConVec[j][i]=Concept_Vector[i][j];
}

void SPKMeans::dump()
{
	int i;

	for (i = 0; i < n_Clusters; i++)
		cout << "Cluster " << i << ": " << ClusterSize[i] << "\n";
//	for (i = 0; i < n_Docs; i++)
//		cout << Cluster[i] << " ";
//	cout << "\n";
	cout << "objective function value :" << GetResult() << "\n";
}


void SPKMeans::compute_cluster_size(int *index)
{
	int i;

	for (i = 0; i < n_Clusters; i++)
		ClusterSize[i] = 0;
	for (i = 0; i < n_sub_Docs; i++)
		ClusterSize[Cluster[index[i]]]++;
}

float SPKMeans::coherence(SparseMatrixDouble *p_Docs, int * index)
{
	int i;
	float value = 0.0;

	if (Alg == SPHERICAL_K_MEANS)
	{
		if (ObjFunc == NON_WEIGHTED_OBJ)
			for (i = 0; i < n_sub_Docs; i++)
			{
			  Inner_Sim[i] = dot_mult(p_Docs, index[i], Concept_Vector[Cluster[index[i]]]);
			  value += Inner_Sim[i];
			}
		else
			;	// ...
	}
	else
	{
		for (i = 0; i < n_sub_Docs; i++)
			value += DocNormSqr[i] + CVNormSqr[Cluster[i]] -
				2 * dot_mult(p_Docs, i, Concept_Vector[Cluster[i]]);
	}

	return value;
}

void SPKMeans::assign_cluster(SparseMatrixDouble *p_Docs, int * index)
{
	int i, j;
/*	VECTOR_double similarity, temp;

	similarity = new double[n_sub_Docs];
	temp = new double[n_sub_Docs]; */

	float temp_sim;
	//int moved = 0;
	int multi = 0;
	int changed = 0;
	//int *tc = new int[n_sub_Docs];
	//for (i = 0; i < n_sub_Docs; i++) tc[i] = Cluster[index[i]];

	// compute the similarity between concept vectors
	// ...
	// only apply to spherical k means
/*	assert(Alg == SPHERICAL_K_MEANS);
	for (i = 0; i < n_Clusters; i++)
	{
		CV_Sim(i, i) = 1.0;
		for (j = 0; j < i; j++)
			CV_Sim(i, j) = dot_mult(Concept_Vector[i], Concept_Vector[j], n_Words);
	} */

	// estimate the similarity matrix
	for (i = 0; i < n_Clusters; i++)
		for (j = 0; j < n_sub_Docs; j++)
			Sim_Mat[i][j] += diff[i];
	for (i = 0; i < n_sub_Docs; i++)
		Sim_Mat[Cluster[index[i]]][i] = Inner_Sim[i];

	// reassign documents
	for (i = 0; i < n_sub_Docs; i++)
	{
		for (j = 0; j < n_Clusters; j++)
			if (j != Cluster[index[i]])
			{
/*				if (CV_Sim(Cluster[i], j) > 2 * Inner_Sim[i] * Inner_Sim[i]- 1)
				{	// ...
					multi++;
					temp_sim = dot_mult(p_Docs, index[i], Concept_Vector[j]);
					if (temp_sim > Inner_Sim[i])
					{
						changed++;
						Cluster[i] = j;
						Inner_Sim[i] = temp_sim;
					}
				} */
				if (Sim_Mat[j][i] > Sim_Mat[Cluster[index[i]]][i])
				{
					multi++;
					temp_sim = dot_mult(p_Docs, index[i], Concept_Vector[j]);
					Sim_Mat[j][i] = temp_sim;
					if (temp_sim > Inner_Sim[i])
					{
						changed++;
						Cluster[index[i]] = j;
						Inner_Sim[i] = temp_sim;
					}
				}
			}
	}

	//cout << multi << ","; //" dot product computation\n";
	//cout << changed << ","; // assignment changes\n";
	//for (i = 0; i < n_sub_Docs; i++) if (tc[i] != Cluster[index[i]]) moved++;
	//cout << moved << ","<<endl;// " documents moved\n"; 

/*
	pDocs->trans_mult(Concept_Vector[0], similarity);
	if (Alg == EUCLIDEAN_K_MEANS)
		for (i = 0; i < n_sub_Docs; i++)	// DocNormSqr is redundant ...
			similarity[i] = DocNormSqr[i] + CVNormSqr[0] - 2 * similarity[i];

	for (i=0; i<n_sub_Docs; i++)
		Cluster[i] = 0;

	for (i=1; i<n_Clusters; i++)
	{
		pDocs->trans_mult(Concept_Vector[i], temp);
		if (Alg == EUCLIDEAN_K_MEANS)
			for (j = 0; j < n_sub_Docs; j++)
				temp[j] = DocNormSqr[j] + CVNormSqr[i]
					- 2 * temp[j];

		for (j = 0; j < n_sub_Docs; j++)
		{
			if (Alg == SPHERICAL_K_MEANS && temp[j] > similarity[j]
				|| Alg == EUCLIDEAN_K_MEANS && temp[j] < similarity[j])
			{
				similarity[j] = temp[j];
				Cluster[j] = i;
			}
		}
	}
*/
}

void SPKMeans::random_perturb_init(SparseMatrixDouble *p_Docs, int * index)
{
	VECTOR_double v, temp_v;
	int i, j;
	float temp_norm;

	v = new float[n_Words];
	temp_v = new float[n_Words];

	for (i = 0; i < n_Words; i++)
		v[i] = 0.0;

	// compute the centroid of document vectors
	for (i = 0; i < n_sub_Docs; i++)
		for (j = p_Docs->col_ptr(index[i]); j < p_Docs->col_ptr(index[i]+1); j++)
			v[p_Docs->row_ind(j)] += p_Docs->val(j);

	if (Alg == SPHERICAL_K_MEANS)
		normalize_vec(v, n_Words);
	else
		for (i = 0; i < n_Words; i++)
			v[i] /= n_sub_Docs;

	// perturb the centroid k times with randomly generated vectors
	for (i = 0; i < n_Clusters; i++)
	{
		//generate a random vector
		for (j = 0; j < n_Words; j++)
			temp_v[j] = rand_gen.GetUniform() - 0.5;
		normalize_vec(temp_v, n_Words);
		temp_norm = Perturb * rand_gen.GetUniform();
		for (j = 0; j < n_Words; j++)
			temp_v[j] *= temp_norm;
		for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = v[j]*(1+ temp_v[j]);
		if (Alg == SPHERICAL_K_MEANS)
			normalize_vec(Concept_Vector[i], n_Words);

		// ...
		/* for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = docs(j, i); */
	}
	delete [] v;
	delete [] temp_v;

/*
#ifdef DEBUG
	cout << "initial concept vectors\n";
	for (i = 0; i < n_Clusters; i++)
	{
		cout << "concept vector " << i << "\n";
		for (j = 0; j < n_Words; j++)
			cout << Concept_Vector[i][j] << " ";
		cout << "\n\n";
	}
#endif */
}

void SPKMeans::h_Iterate(SparseMatrixDouble *p_Docs, int * index)
{
	float temp_result;
	int i, j;
	
	
	// normalize document matrix if using spherical k means algorithm
	// else compute norms of document vectors
	
	//h_encode_mat(p_Docs, index, n_sub_Docs, EncodeScheme);
	
	random_perturb_init(p_Docs, index);
		

			
	// initial assignment, all documents are assigned to cluster 0
	for (i = 0; i < n_sub_Docs; i++)
	{
		Cluster[index[i]] = 0;
		Inner_Sim[i] = dot_mult(p_Docs, index[i], Concept_Vector[0]);
	}

	for (i = 0; i < n_Clusters; i++)
		p_Docs->h_trans_mult(Concept_Vector[i], index , n_sub_Docs, Sim_Mat[i]);
	for (i = 0; i < n_Clusters; i++)
		diff[i] = 0.0;

	/* iterate until the stopping criterion is met
	   the stopping criterion adopted here is to stop
	   if the "change" in objective function after an iteration
	   of the algorithm is less than a certain threshold. */
	Result = 0.0;
	n_Iters = 0;

	do
	{
		assign_cluster(p_Docs, index);
		compute_cluster_size(index);

		temp_result = Result;
		n_Iters++;


		// save old concept vectors
		for (i = 0; i < n_Clusters; i++)
			for (j = 0; j < n_Words; j++)
				old_CV[i][j] = Concept_Vector[i][j];

		// reset concept vectors
		for (i = 0; i < n_Clusters; i++)
			for (j = 0; j < n_Words; j++)
				Concept_Vector[i][j] = 0.0;

		// compute new concept vectors
		for (i = 0; i < n_sub_Docs; i++)
		{
			assert(Cluster[index[i]] >=0 && Cluster[index[i]] < n_Clusters);
			for (j = p_Docs->col_ptr(index[i]); j < p_Docs->col_ptr(index[i]+1); j++)
				Concept_Vector[Cluster[index[i]]][p_Docs->row_ind(j)]
					+= p_Docs->val(j);
		}

		if (Alg == SPHERICAL_K_MEANS)
			for (i = 0; i < n_Clusters; i++)
				normalize_vec(Concept_Vector[i], n_Words);
		else
		{
			for (i = 0; i < n_Clusters; i++)
			{
				CVNormSqr[i] = 0;
				if (ClusterSize[i] != 0)
					for (j = 0; j < n_Words; j++)
					{
						Concept_Vector[i][j] /= ClusterSize[i];
						CVNormSqr[i] += Concept_Vector[i][j]
							* Concept_Vector[i][j];
					}
			}
		}
		
		// compute the difference between old and new concept vectors
		for (i = 0; i < n_Clusters; i++)
		{
			diff[i] = 0.0;
			for (j = 0; j < n_Words; j++)
				diff[i] += (old_CV[i][j] - Concept_Vector[i][j]) *
					(old_CV[i][j] - Concept_Vector[i][j]);
			diff[i] = sqrt(diff[i]);
		}

		// compute new objective function
		Result = coherence(p_Docs, index);
		//cout<<Result<<endl;
		if (DumpCSize) dump();

	} while (//(temp_result > Result || Alg != EUCLIDEAN_K_MEANS) &&
		(fabs(temp_result - Result) > Epsilon*Result));
	
}

void SPKMeans::Iterate(SparseMatrixDouble *p_Docs, int level, char* hierachy)
{
  float temp_result;
  int i, j;
  //clock_t start_clock, finish_clock;
  
  // normalize document matrix if using spherical k means algorithm
  // else compute norms of document vectors
  if (Alg == SPHERICAL_K_MEANS)
    normalize_mat(p_Docs);
  else
    {
      for (i = 0; i < n_Docs; i++)
	{
	  DocNormSqr[i] = 0;
	  for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
	    DocNormSqr[i] += p_Docs->val(j) * p_Docs->val(j);
	}
    }
  
  switch (Init_Method)
    {
    case SAMPLE_INIT:
      random_perturb_init(p_Docs);
      //refined_partition(p_Docs,n_Samples);
      break;
    case RANDOM_PERTURB_INIT:
      random_perturb_init(p_Docs);
      break;
    case RANDOM_INIT:
      random_init(p_Docs);
      if (Alg == EUCLIDEAN_K_MEANS)	// should be moved into the concept vector initialization
	{
	  for (i = 0; i < n_Clusters; i++)
	    {
	      CVNormSqr[i] = 0;
	      for (j = 0; j < n_Words; j++)
		CVNormSqr[i] += Concept_Vector[i][j] * Concept_Vector[i][j];
	    }
	}
      break;
    default:
      break;
    }
  
  
  // initial assignment, all documents are assigned to cluster 0
  for (i = 0; i < n_Docs; i++)
    {
      Cluster[i] = 0;
      Inner_Sim[i] = dot_mult(p_Docs, i, Concept_Vector[0]);
    }

  for (i = 0; i < n_Clusters; i++)
    p_Docs->trans_mult(Concept_Vector[i], Sim_Mat[i]);
  for (i = 0; i < n_Clusters; i++)
    diff[i] = 0.0;
  
  
  
  /* iterate until the stopping criterion is met
     the stopping criterion adopted here is to stop
     if the "change" in objective function after an iteration
     of the algorithm is less than a certain threshold. */
  Result = 0.0;
  n_Iters = 0;
  //	pDocs->prepare_trans_mult();
  //start_clock = clock();
  do
    {
      assign_cluster(p_Docs);
      compute_cluster_size();
      
      temp_result = Result;
      n_Iters++;
      
      
      // save old concept vectors
      for (i = 0; i < n_Clusters; i++)
	for (j = 0; j < n_Words; j++)
	  old_CV[i][j] = Concept_Vector[i][j];
      
      // reset concept vectors
      for (i = 0; i < n_Clusters; i++)
	for (j = 0; j < n_Words; j++)
	  Concept_Vector[i][j] = 0.0;
      
      // compute new concept vectors
      for (i = 0; i < n_Docs; i++)
	{
	  assert(Cluster[i] >=0 && Cluster[i] < n_Clusters);
	  for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
	    Concept_Vector[Cluster[i]][p_Docs->row_ind(j)]
	      += p_Docs->val(j);
	}
      
      if (Alg == SPHERICAL_K_MEANS)
	for (i = 0; i < n_Clusters; i++)
	  normalize_vec(Concept_Vector[i], n_Words);
      else
	{
	  for (i = 0; i < n_Clusters; i++)
	    {
	      CVNormSqr[i] = 0;
	      if (ClusterSize[i] != 0)
		for (j = 0; j < n_Words; j++)
		  {
		    Concept_Vector[i][j] /= ClusterSize[i];
			    CVNormSqr[i] += Concept_Vector[i][j]
			      * Concept_Vector[i][j];
		  }
	    }
	}
      
      // compute the difference between old and new concept vectors
      for (i = 0; i < n_Clusters; i++)
	{
	  diff[i] = 0.0;
	  for (j = 0; j < n_Words; j++)
	    diff[i] += (old_CV[i][j] - Concept_Vector[i][j]) *
	      (old_CV[i][j] - Concept_Vector[i][j]);
	  diff[i] = sqrt(diff[i]);
	}
      
      // compute new objective function
      Result = coherence(p_Docs);
      //cout<<Result<<endl;
      if (DumpCSize) dump();
      
    } while (//(temp_result > Result || Alg != EUCLIDEAN_K_MEANS) &&
	     (fabs(temp_result - Result) > Epsilon*Result));
  
  /**************************************/
  /* h_clustering adds here     */
  /**************************************/
	
  int * index = new int[n_Docs];
  int * h_ClusterSize = new int[int_power(n_Clusters, level)];
  memory_consume+= (n_Docs+int_power(n_Clusters, level)) * sizeof(int);

  cout<<"Level 0 ..."<<endl;
  cout<<"Original Size: "<<n_Docs<<endl<<endl;
  char hierachy_name[256];
  char name_post[256];
  
  strcpy(hierachy_name, hierachy);
  sprintf(name_post, "_%d_%d", level,n_Clusters);
  strcat(hierachy_name, name_post);
  std::ofstream Hierachy(hierachy_name);
  Hierachy<<0;
  for(int k=0; k<n_Clusters;k++)
    Hierachy<<' '<<k;
  Hierachy<<endl<<endl;
  int * pointer = new int[int_power(n_Clusters, level)];
  for (i=1; i<level; i++)
    {
      //group_index(index, h_ClusterSize);
      
      for(int k=0; k< int_power(n_Clusters, i); k++)
	h_ClusterSize[k]=0;
      
      for(int k=0; k<n_Docs; k++)
	h_ClusterSize[Cluster[k]] ++;
      
      // int * pointer = new int[int_power(n_Clusters, level)];
      pointer[0]= 0;
      
      cout<<"Level "<<i<<" ..."<<endl;
      for(int k=0; k< int_power(n_Clusters, i); k++)
	cout <<"h_ClusterSize["<<k<<"]="<<h_ClusterSize[k]<<endl;
      
      cout<<endl;
      
      for(int k=1; k< int_power(n_Clusters, i); k++)
	pointer[k]= h_ClusterSize[k-1]+pointer[k-1];
      
      for(int k=0; k<n_Docs; k++)
	{
	  index[pointer[Cluster[k]]] = k;
	  pointer[Cluster[k]] ++;
	}
      /*for(int k=0; k<1000;k++)
	cout <<Cluster[index[k]]<<" ";
	cout<<endl;*/
      
      /* reset pointer[] */
      pointer[0]= 0;
      
      for(int k=1; k< int_power(n_Clusters, i); k++)
	pointer[k]= h_ClusterSize[k-1]+pointer[k-1];
      
      
      
      for (j=0; j<int_power(n_Clusters, i); j++)
	{
	  if (h_ClusterSize[j] !=0)
	    {
	      Hierachy<<j;
	      for(int k=0; k<n_Clusters;k++)
		Hierachy<<' '<<k+j*n_Clusters;
	      Hierachy<<endl;
	      
	      n_sub_Docs = h_ClusterSize[j];
	      cout <<"Branch "<<j<<" ..."<<endl;
	      
	      
	      h_Iterate(p_Docs, &(index[pointer[j]]));
	      for(int k =0; k< h_ClusterSize[j]; k++)
		Cluster[index[pointer[j]+k]] += j*n_Clusters;
	    }
	  else
	    {
	      Hierachy<<j<<endl;
	    }
	}
      Hierachy<<endl;
    }
  
  delete [] pointer;
  Hierachy.close();

  for(int k=0; k< int_power(n_Clusters, i); k++)
    h_ClusterSize[k]=0;
  
  for(int k=0; k<n_Docs; k++)
    h_ClusterSize[Cluster[k]] ++;
  
  cout<<"Level "<<i<<" ..."<<endl;
  for(int k=0; k< int_power(n_Clusters, i); k++)
    cout <<"h_ClusterSize["<<k<<"]="<<h_ClusterSize[k]<<endl;
  
  
  
  
  /**************************************/
  /* h_clustering adds here     */
  /**************************************/
  
  //finish_clock = clock();
  //cout<<"Core kmeans (routine) time: "<<(finish_clock - start_clock)/1e6<<endl;
  /*
  //delete the concept vectors for the last node
  for (int i = 0; i < n_Clusters; i++)
    delete[] Concept_Vector[i];
  delete[] Concept_Vector;
  
  //generate the concept vector for n_Cluster ^ level clusters
  Concept_Vector = new VECTOR_double[int_power(n_Clusters,level)];
  for (int i = 0; i < int_power(n_Clusters,level); i++)
    Concept_Vector[i] = new float[n_Words];
  
  

  // compute new concept vectors
  for (i = 0; i < n_Docs; i++)
    {
      assert(Cluster[i] >=0 && Cluster[i] < int_power(n_Clusters,level));
      for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
	Concept_Vector[Cluster[i]][p_Docs->row_ind(j)] += p_Docs->val(j);
    }
  
  if (Alg == SPHERICAL_K_MEANS)
    for (i = 0; i < int_power(n_Clusters,level); i++)
      normalize_vec(Concept_Vector[i], n_Words);
  
  float *word_den = new float[int_power(n_Clusters,level)];
  for (i = 0; i < int_power(n_Clusters,level); i++)
    {
      word_den[i] = 0.0;
      for (j = 0; j < n_Words; j++)
	if (Concept_Vector[i][j] != 0) word_den[i] += 1.0/n_Words;
    }
  
  cout << "word density of concept vectors: ";
  for (i = 0; i < int_power(n_Clusters,level); i++)
    cout << word_den[i] << " ";
  cout << "\n";
  
  Result = coherence(p_Docs);
  cout<<Result<<endl;
  */
}


void SPKMeans::Iterate(SparseMatrixDouble *p_Docs, char * filename)
{
  float temp_result;
  int i, j;
  clock_t start_clock, finish_clock;
  
  // normalize document matrix if using spherical k means algorithm
  // else compute norms of document vectors
  if (Alg == SPHERICAL_K_MEANS)
    normalize_mat(p_Docs);
  else
    {
      for (i = 0; i < n_Docs; i++)
	{
	  DocNormSqr[i] = 0;
	  for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
	    DocNormSqr[i] += p_Docs->val(j) * p_Docs->val(j);
	}
    }

  std::ifstream gpfile(filename);
  gpfile>>i>>j;

  if(i!=n_Docs)
  {
    cout<<"Wrong number of document!!!"<<endl;
    exit(1);
  }
  
  for (i=0; i<n_Docs; i++)
    gpfile>>Cluster[i];

  // reset concept vectors
  for (i = 0; i < n_Clusters; i++)
    for (j = 0; j < n_Words; j++)
      Concept_Vector[i][j] = 0.0;

  // compute new concept vectors
  for (i = 0; i < n_Docs; i++)
    {
      assert(Cluster[i] >=0 && Cluster[i] < n_Clusters);
      for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
	Concept_Vector[Cluster[i]][p_Docs->row_ind(j)]
					+= p_Docs->val(j);
    }

  if (Alg == SPHERICAL_K_MEANS)
    for (i = 0; i < n_Clusters; i++)
      normalize_vec(Concept_Vector[i], n_Words);
  else
    {
      for (i = 0; i < n_Clusters; i++)
	{
	  CVNormSqr[i] = 0;
	  if (ClusterSize[i] != 0)
	    for (j = 0; j < n_Words; j++)
	      {
		Concept_Vector[i][j] /= ClusterSize[i];
		CVNormSqr[i] += Concept_Vector[i][j]
		  * Concept_Vector[i][j];
	      }
	}
    }
  
  // initial assignment
  for (i = 0; i < n_Docs; i++)
    Inner_Sim[i] = dot_mult(p_Docs, i, Concept_Vector[Cluster[i]]);
  for (i = 0; i < n_Clusters; i++)
    p_Docs->trans_mult(Concept_Vector[i], Sim_Mat[i]);
  
  for (i = 0; i < n_Clusters; i++)
    diff[i] = 0.0;
  
  /* iterate until the stopping criterion is met
     the stopping criterion adopted here is to stop
     if the "change" in objective function after an iteration
     of the algorithm is less than a certain threshold. */
  Result = 0.0;
  n_Iters = 0;
  cout<<"Objective function values are: "<<endl;
  //	pDocs->prepare_trans_mult();
  start_clock = clock();
  do
    {
      assign_cluster(p_Docs);
      compute_cluster_size();
      
      temp_result = Result;
      n_Iters++;
      
      
      // save old concept vectors
      for (i = 0; i < n_Clusters; i++)
	for (j = 0; j < n_Words; j++)
	  old_CV[i][j] = Concept_Vector[i][j];
      
      // reset concept vectors
      for (i = 0; i < n_Clusters; i++)
	for (j = 0; j < n_Words; j++)
	  Concept_Vector[i][j] = 0.0;
      
      // compute new concept vectors
      for (i = 0; i < n_Docs; i++)
	{
	  assert(Cluster[i] >=0 && Cluster[i] < n_Clusters);
	  for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
	    Concept_Vector[Cluster[i]][p_Docs->row_ind(j)]
					+= p_Docs->val(j);
	}
      
      if (Alg == SPHERICAL_K_MEANS)
	for (i = 0; i < n_Clusters; i++)
	  normalize_vec(Concept_Vector[i], n_Words);
      else
	{
	  for (i = 0; i < n_Clusters; i++)
	    {
	      CVNormSqr[i] = 0;
	      if (ClusterSize[i] != 0)
		for (j = 0; j < n_Words; j++)
		  {
		    Concept_Vector[i][j] /= ClusterSize[i];
		    CVNormSqr[i] += Concept_Vector[i][j]
		      * Concept_Vector[i][j];
		  }
	    }
	}
      
      // compute the difference between old and new concept vectors
      for (i = 0; i < n_Clusters; i++)
	{
	  diff[i] = 0.0;
	  for (j = 0; j < n_Words; j++)
	    diff[i] += (old_CV[i][j] - Concept_Vector[i][j]) *
	      (old_CV[i][j] - Concept_Vector[i][j]);
	  diff[i] = sqrt(diff[i]);
	}
      
      // compute new objective function
      Result = coherence(p_Docs);
      cout<<Result<<endl;
      if (DumpCSize) dump();
      
    } while (//(temp_result > Result || Alg != EUCLIDEAN_K_MEANS) &&
	     (fabs(temp_result - Result) > Epsilon));
 
  finish_clock = clock();
  cout<<"Core kmeans (routine) time: "<<(finish_clock - start_clock)/1e6<<endl;
  float *word_den = new float[n_Clusters];
  for (i = 0; i < n_Clusters; i++)
    {
      word_den[i] = 0.0;
      for (j = 0; j < n_Words; j++)
	if (Concept_Vector[i][j] != 0) word_den[i] += 1.0/n_Words;
    }
  
  cout << "word density of concept vectors: ";
  for (i = 0; i < n_Clusters; i++)
    cout << word_den[i] << " ";
  cout << "\n";
}

void SPKMeans::Iterate(DenseMatrixDouble *p_Docs)
{
	float temp_result;
	int i, j;
	clock_t start_clock, finish_clock;

	// normalize document matrix if using spherical k means algorithm
	// else compute norms of document vectors
	if (Alg == SPHERICAL_K_MEANS)
		normalize_mat(p_Docs);
	else
	{
		for (i = 0; i <n_Docs; i++)
		{
			DocNormSqr[i] = 0;
			for (j = 0; j<p_Docs->GetNumRow(); j++)
				DocNormSqr[i] += p_Docs->val(j, i) * p_Docs->val(j, i);
		}
	}

	switch (Init_Method)
	{
	case SAMPLE_INIT:
		random_perturb_init(p_Docs);
		//refined_partition(p_Docs, n_Samples);
		break;
	case RANDOM_PERTURB_INIT:
		random_perturb_init(p_Docs);
		break;
	case RANDOM_INIT:
		random_init(p_Docs);
		if (Alg == EUCLIDEAN_K_MEANS)	// should be moved into the concept vector initialization
		{
			for (i = 0; i < n_Clusters; i++)
			{
				CVNormSqr[i] = 0;
				for (j = 0; j < n_Words; j++)
					CVNormSqr[i] += Concept_Vector[i][j] * Concept_Vector[i][j];
			}
		}
		break;
	default:
		break;
	}

	
	// initial assignment, all documents are assigned to cluster 0
	for (i = 0; i < n_Docs; i++)
	{
		Cluster[i] = 0;
		Inner_Sim[i] = dot_mult(p_Docs, i, Concept_Vector[0]);
	}

	for (i = 0; i < n_Clusters; i++)
		p_Docs->trans_mult(Concept_Vector[i], Sim_Mat[i]);
	for (i = 0; i < n_Clusters; i++)
		diff[i] = 0.0;

	/* iterate until the stopping criterion is met
	   the stopping criterion adopted here is to stop
	   if the "change" in objective function after an iteration
	   of the algorithm is less than a certain threshold. */
	Result = 0.0;
	n_Iters = 0;

//	pDocs->prepare_trans_mult();
	start_clock = clock();
	
	do
	{
		assign_cluster(p_Docs);
		compute_cluster_size();

		temp_result = Result;
		n_Iters++;


		// save old concept vectors
		for (i = 0; i < n_Clusters; i++)
			for (j = 0; j < n_Words; j++)
				old_CV[i][j] = Concept_Vector[i][j];

		// reset concept vectors
		for (i = 0; i < n_Clusters; i++)
			for (j = 0; j < n_Words; j++)
				Concept_Vector[i][j] = 0.0;

		// compute new concept vectors
		for (i = 0; i < n_Docs; i++)
		{
			assert(Cluster[i] >=0 && Cluster[i] < n_Clusters);
			for (j =0; j< p_Docs->GetNumRow ();  j++)
				Concept_Vector[Cluster[i]][j]
					+= p_Docs->val(j, i);
		}

		if (Alg == SPHERICAL_K_MEANS)
			for (i = 0; i < n_Clusters; i++)
				normalize_vec(Concept_Vector[i], n_Words);
		else
		{
			for (i = 0; i < n_Clusters; i++)
			{
				CVNormSqr[i] = 0;
				if (ClusterSize[i] != 0)
					for (j = 0; j < n_Words; j++)
					{
						Concept_Vector[i][j] /= ClusterSize[i];
						CVNormSqr[i] += Concept_Vector[i][j]
							* Concept_Vector[i][j];
					}
			}
		}
		
		// compute the difference between old and new concept vectors
		for (i = 0; i < n_Clusters; i++)
		{
			diff[i] = 0.0;
			for (j = 0; j < n_Words; j++)
				diff[i] += (old_CV[i][j] - Concept_Vector[i][j]) *
					(old_CV[i][j] - Concept_Vector[i][j]);
			diff[i] = sqrt(diff[i]);
		}

		// compute new objective function
		Result = coherence(p_Docs);

		if (DumpCSize) dump();

	} while (//(temp_result > Result || Alg != EUCLIDEAN_K_MEANS) &&
		(fabs(temp_result - Result) > Epsilon));
	finish_clock = clock();
	cout<<"Core kmeans (routine) time: "<<(finish_clock - start_clock)/1e6<<endl;
	float *word_den = new float[n_Clusters];
	for (i = 0; i < n_Clusters; i++)
	{
		word_den[i] = 0.0;
		for (j = 0; j < n_Words; j++)
			if (Concept_Vector[i][j] != 0) word_den[i] += 1.0/n_Words;
	}

	cout << "word density of concept vectors: ";
	for (i = 0; i < n_Clusters; i++)
		cout << word_den[i] << " ";
	cout << "\n";
}


int SPKMeans::h_SortCluster(int *cluster_size, int n_Clusters)
{
  int *cluster_label = new int[n_Clusters];
	int *new_label = new int[n_Clusters];
	int i, j, tmp_size, tmp_label;
	bool flag;

	for (i = 0; i < n_Clusters; i++)
	{
		cluster_size[i] = 0;
		cluster_label[i] = i;
	}
	for (i = 0; i < n_Docs; i++)
		cluster_size[Cluster[i]]++;

	// ugly sorting
	for (i = 0; i < n_Clusters-1; i++)
	{
		flag = true;
		for (j = 0; j < n_Clusters-i-1; j++)
			if (cluster_size[j+1] < cluster_size[j])
			{
				flag = false;
				tmp_size = cluster_size[j+1];
				tmp_label = cluster_label[j+1];
				cluster_size[j+1] = cluster_size[j];
				cluster_label[j+1] = cluster_label[j];
				cluster_size[j] = tmp_size;
				cluster_label[j] = tmp_label;
			}
		if (flag) break;
	}

	// eliminate the empty clusters
	int emptyCluster=0;
	i=0;
	while(cluster_size [i++] ==0)
	  emptyCluster++;
	    
	WordCluster = new int[n_Words];
	int index;
	float max;

	for (j = 0; j < n_Words; j++)
	  {
	    max = Concept_Vector[0][j];
	    index =0;

	    for (i = 1; i < n_Clusters; i++)
	      if (Concept_Vector[i][j] > max) 
		{
		  max = Concept_Vector[i][j];
		  index =i;
		}
	    WordCluster[j]=index;
	  }
	
	// modify Cluster array, also ugly
	for (i = 0; i < n_Clusters; i++)
		new_label[cluster_label[i]] = i;

	for (i = 0; i < n_Docs; i++)
	  Cluster[i] = new_label[Cluster[i]]-emptyCluster;

	for (i = 0; i < n_Words; i++)
	  WordCluster[i] = new_label[WordCluster[i]]-emptyCluster;

	return emptyCluster;
}

int SPKMeans::SortCluster(int *cluster_size )
{
  //int *cluster_size = new int[n_Clusters];
	int *cluster_label = new int[n_Clusters];
	int *new_label = new int[n_Clusters];
	int i, j, tmp_size, tmp_label;
	bool flag;

	for (i = 0; i < n_Clusters; i++)
	{
		cluster_size[i] = 0;
		cluster_label[i] = i;
	}
	for (i = 0; i < n_Docs; i++)
		cluster_size[Cluster[i]]++;

	// ugly sorting
	for (i = 0; i < n_Clusters-1; i++)
	{
		flag = true;
		for (j = 0; j < n_Clusters-i-1; j++)
			if (cluster_size[j+1] < cluster_size[j])
			{
				flag = false;
				tmp_size = cluster_size[j+1];
				tmp_label = cluster_label[j+1];
				cluster_size[j+1] = cluster_size[j];
				cluster_label[j+1] = cluster_label[j];
				cluster_size[j] = tmp_size;
				cluster_label[j] = tmp_label;
			}
		if (flag) break;
	}

	// eliminate the empty clusters
	int emptyCluster=0;
	i=0;
	while(cluster_size [i++] ==0)
	  emptyCluster++;
	    
	WordCluster = new int[n_Words];
	int index;
	float max;

	for (j = 0; j < n_Words; j++)
	  {
	    max = Concept_Vector[0][j];
	    index =0;

	    for (i = 1; i < n_Clusters; i++)
	      if (Concept_Vector[i][j] > max) 
		{
		  max = Concept_Vector[i][j];
		  index =i;
		}
	    WordCluster[j]=index;
	  }
	
	// modify Cluster array, also ugly
	for (i = 0; i < n_Clusters; i++)
		new_label[cluster_label[i]] = i;

	for (i = 0; i < n_Docs; i++)
	  Cluster[i] = new_label[Cluster[i]]-emptyCluster;

	for (i = 0; i < n_Words; i++)
	  WordCluster[i] = new_label[WordCluster[i]]-emptyCluster;

	return emptyCluster;
}

void SPKMeans::wordCluster(char *output_matrix, int level)
{
  char browserfilepost[128];
  //double max; 
  int j, i;
  WordCluster = new int[n_Words];
	int index;
	float max;

	for (j = 0; j < n_Words; j++)
	  {
	    max = Concept_Vector[0][j];
	    index =0;

	    for (i = 1; i < n_Clusters; i++)
	      if (Concept_Vector[i][j] > max) 
		{
		  max = Concept_Vector[i][j];
		  index =i;
		}
	    WordCluster[j]=index;
	  }

  sprintf(browserfilepost, "_wordtoclus_%d_%d", level, n_Clusters);
  strcat(output_matrix, browserfilepost);
  std::ofstream o_m(output_matrix);
  
  o_m<< n_Words <<endl;

  for (j = 0; j < n_Words; j++)
    o_m<<WordCluster[j]<<endl;
 
  o_m.close();
    
}

void SPKMeans::compute_cluster_size()
{
	int i;

	for (i = 0; i < n_Clusters; i++)
		ClusterSize[i] = 0;
	for (i = 0; i < n_Docs; i++)
		ClusterSize[Cluster[i]]++;
}

float SPKMeans::coherence(SparseMatrixDouble *p_Docs)
{
  int i;
  float value = 0.0;
  
  if (Alg == SPHERICAL_K_MEANS)
    {
      if (ObjFunc == NON_WEIGHTED_OBJ)
	for (i = 0; i < n_Docs; i++)
	  {
	    Inner_Sim[i] = dot_mult(p_Docs, i, Concept_Vector[Cluster[i]]);
	    value += Inner_Sim[i];
	  }
      else
	;	// ...
    }
  else
    {
      for (i = 0; i < n_Docs; i++)
	value += DocNormSqr[i] + CVNormSqr[Cluster[i]] -
	  2 * dot_mult(p_Docs, i, Concept_Vector[Cluster[i]]);
    }
  
  return value;
}

float SPKMeans::coherence(DenseMatrixDouble *p_Docs)
{
	int i;
	float value = 0.0;

	if (Alg == SPHERICAL_K_MEANS)
	{
		if (ObjFunc == NON_WEIGHTED_OBJ)
			for (i = 0; i < n_Docs; i++)
			{
				Inner_Sim[i] = dot_mult(p_Docs, i, Concept_Vector[Cluster[i]]);
				value += Inner_Sim[i];
			}
		else
			;	// ...
	}
	else
	{
		for (i = 0; i < n_Docs; i++)
			value += DocNormSqr[i] + CVNormSqr[Cluster[i]] -
				2 * dot_mult(p_Docs, i, Concept_Vector[Cluster[i]]);
	}

	return value;
}

void SPKMeans::assign_cluster(SparseMatrixDouble *p_Docs)
{
	int i, j;
/*	VECTOR_double similarity, temp;

	similarity = new double[n_Docs];
	temp = new double[n_Docs]; */

	float temp_sim;
	int moved = 0;
	int multi = 0;
	int changed = 0;
	int *tc = new int[n_Docs];
	for (i = 0; i < n_Docs; i++) tc[i] = Cluster[i];

	// compute the similarity between concept vectors
	// ...
	// only apply to spherical k means
/*	assert(Alg == SPHERICAL_K_MEANS);
	for (i = 0; i < n_Clusters; i++)
	{
		CV_Sim(i, i) = 1.0;
		for (j = 0; j < i; j++)
			CV_Sim(i, j) = dot_mult(Concept_Vector[i], Concept_Vector[j], n_Words);
	} */

	// estimate the similarity matrix
	for (i = 0; i < n_Clusters; i++)
		for (j = 0; j < n_Docs; j++)
			Sim_Mat[i][j] += diff[i];
	for (i = 0; i < n_Docs; i++)
		Sim_Mat[Cluster[i]][i] = Inner_Sim[i];

	// reassign documents
	for (i = 0; i < n_Docs; i++)
	{
		for (j = 0; j < n_Clusters; j++)
			if (j != Cluster[i])
			{
/*				if (CV_Sim(Cluster[i], j) > 2 * Inner_Sim[i] * Inner_Sim[i]- 1)
				{	// ...
					multi++;
					temp_sim = dot_mult(p_Docs, i, Concept_Vector[j]);
					if (temp_sim > Inner_Sim[i])
					{
						changed++;
						Cluster[i] = j;
						Inner_Sim[i] = temp_sim;
					}
				} */
				if (Sim_Mat[j][i] > Sim_Mat[Cluster[i]][i])
				{
					multi++;
					temp_sim = dot_mult(p_Docs, i, Concept_Vector[j]);
					Sim_Mat[j][i] = temp_sim;
					if (temp_sim > Inner_Sim[i])
					{
						changed++;
						Cluster[i] = j;
						Inner_Sim[i] = temp_sim;
					}
				}
			}
	}

	//cout << multi << ","; //" dot product computation\n";
	//cout << changed << ","; // assignment changes\n";
	for (i = 0; i < n_Docs; i++) if (tc[i] != Cluster[i]) moved++;
	//cout << moved << ","<<endl;// " documents moved\n";
/*
	pDocs->trans_mult(Concept_Vector[0], similarity);
	if (Alg == EUCLIDEAN_K_MEANS)
		for (i = 0; i < n_Docs; i++)	// DocNormSqr is redundant ...
			similarity[i] = DocNormSqr[i] + CVNormSqr[0] - 2 * similarity[i];

	for (i=0; i<n_Docs; i++)
		Cluster[i] = 0;

	for (i=1; i<n_Clusters; i++)
	{
		pDocs->trans_mult(Concept_Vector[i], temp);
		if (Alg == EUCLIDEAN_K_MEANS)
			for (j = 0; j < n_Docs; j++)
				temp[j] = DocNormSqr[j] + CVNormSqr[i]
					- 2 * temp[j];

		for (j = 0; j < n_Docs; j++)
		{
			if (Alg == SPHERICAL_K_MEANS && temp[j] > similarity[j]
				|| Alg == EUCLIDEAN_K_MEANS && temp[j] < similarity[j])
			{
				similarity[j] = temp[j];
				Cluster[j] = i;
			}
		}
	}
*/
}

void SPKMeans::assign_cluster(DenseMatrixDouble *p_Docs)
{
	int i, j;

	float temp_sim;
	int moved = 0;
	int multi = 0;
	int changed = 0;
	int *tc = new int[n_Docs];
	for (i = 0; i < n_Docs; i++) tc[i] = Cluster[i];

	// compute the similarity between concept vectors
	// ...
	// only apply to spherical k means

	
	// estimate the similarity matrix
	for (i = 0; i < n_Clusters; i++)
		for (j = 0; j < n_Docs; j++)
			Sim_Mat[i][j] += diff[i];
	for (i = 0; i < n_Docs; i++)
		Sim_Mat[Cluster[i]][i] = Inner_Sim[i];

	// reassign documents
	for (i = 0; i < n_Docs; i++)
	{
		for (j = 0; j < n_Clusters; j++)
			if (j != Cluster[i])
			{

				if (Sim_Mat[j][i] > Sim_Mat[Cluster[i]][i])
				{
					multi++;
					
					temp_sim = dot_mult(p_Docs, i, Concept_Vector[j]);
					
					Sim_Mat[j][i] = temp_sim;
					if (temp_sim > Inner_Sim[i])
					{
						changed++;
						Cluster[i] = j;
						Inner_Sim[i] = temp_sim;
					}
				}
			}
	}

	cout << multi << " dot product computation\n";
	cout << changed << " assignment changes\n";
	for (i = 0; i < n_Docs; i++) if (tc[i] != Cluster[i]) moved++;
	cout << moved << " documents moved\n"; 
/*
	pDocs->trans_mult(Concept_Vector[0], similarity);
	if (Alg == EUCLIDEAN_K_MEANS)
		for (i = 0; i < n_Docs; i++)	// DocNormSqr is redundant ...
			similarity[i] = DocNormSqr[i] + CVNormSqr[0] - 2 * similarity[i];

	for (i=0; i<n_Docs; i++)
		Cluster[i] = 0;

	for (i=1; i<n_Clusters; i++)
	{
		pDocs->trans_mult(Concept_Vector[i], temp);
		if (Alg == EUCLIDEAN_K_MEANS)
			for (j = 0; j < n_Docs; j++)
				temp[j] = DocNormSqr[j] + CVNormSqr[i]
					- 2 * temp[j];

		for (j = 0; j < n_Docs; j++)
		{
			if (Alg == SPHERICAL_K_MEANS && temp[j] > similarity[j]
				|| Alg == EUCLIDEAN_K_MEANS && temp[j] < similarity[j])
			{
				similarity[j] = temp[j];
				Cluster[j] = i;
			}
		}
	}
*/
}

// select n_Clusters document vectors as concept vectors
void SPKMeans::random_init(SparseMatrixDouble *p_Docs)
{
	int i, j;
	int *selected;

	selected = new int[n_Clusters];

	// select document vectors
	for (i = 0; i < n_Clusters; i++)
	{
		while (true)
		{
			selected[i] = rand_gen.GetUniformInt(n_Docs);
			j = 0;
			while (j < i && selected[j] != selected[i])
				j++;
			if (j == i)	// this document vector has not been selected
				break;
		}
	}

	// copy document vectors to concept vectors
	for (i = 0; i < n_Clusters; i++)
		for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = (*p_Docs)(j, selected[i]);
}

void SPKMeans::random_init(DenseMatrixDouble *p_Docs)
{
	int i, j;
	int *selected;

	selected = new int[n_Clusters];

	// select document vectors
	for (i = 0; i < n_Clusters; i++)
	{
		while (true)
		{
			selected[i] = rand_gen.GetUniformInt(n_Docs);
			j = 0;
			while (j < i && selected[j] != selected[i])
				j++;
			if (j == i)	// this document vector has not been selected
				break;
		}
	}

	// copy document vectors to concept vectors
	for (i = 0; i < n_Clusters; i++)
		for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = p_Docs->val(selected[i], j);
}

// nor verified with classic k means ...
void SPKMeans::random_perturb_init(SparseMatrixDouble *p_Docs)
{
	VECTOR_double v, temp_v;
	int i, j;
	float temp_norm;

	v = new float[n_Words];
	temp_v = new float[n_Words];

	for (i = 0; i < n_Words; i++)
		v[i] = 0.0;

	// compute the centroid of document vectors
	for (i = 0; i < n_Docs; i++)
		for (j = p_Docs->col_ptr(i); j < p_Docs->col_ptr(i+1); j++)
			v[p_Docs->row_ind(j)] += p_Docs->val(j);

	if (Alg == SPHERICAL_K_MEANS)
		normalize_vec(v, n_Words);
	else
		for (i = 0; i < n_Words; i++)
			v[i] /= n_Docs;

	// perturb the centroid k times with randomly generated vectors
	for (i = 0; i < n_Clusters; i++)
	{
		//generate a random vector
		for (j = 0; j < n_Words; j++)
			temp_v[j] = rand_gen.GetUniform() - 0.5;
		normalize_vec(temp_v, n_Words);
		temp_norm = Perturb * rand_gen.GetUniform();
		for (j = 0; j < n_Words; j++)
			temp_v[j] *= temp_norm;
		for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = v[j]*(1+ temp_v[j]);
		if (Alg == SPHERICAL_K_MEANS)
			normalize_vec(Concept_Vector[i], n_Words);

		// ...
		/* for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = docs(j, i); */
	}

/*
#ifdef DEBUG
	cout << "initial concept vectors\n";
	for (i = 0; i < n_Clusters; i++)
	{
		cout << "concept vector " << i << "\n";
		for (j = 0; j < n_Words; j++)
			cout << Concept_Vector[i][j] << " ";
		cout << "\n\n";
	}
#endif */
}

void SPKMeans::random_perturb_init(DenseMatrixDouble *p_Docs)
{
	VECTOR_double v, temp_v;
	int i, j;
	float temp_norm;

	v = new float[n_Words];
	temp_v = new float[n_Words];
	
	for (i = 0; i < n_Words; i++)
		v[i] = 0.0;

	// compute the centroid of document vectors
	for (i = 0; i < p_Docs->GetNumRow (); i++)
		for (j =0; j< p_Docs->GetNumCol (); j++)
			v[i] += p_Docs->val(i, j);

	if (Alg == SPHERICAL_K_MEANS)
		normalize_vec(v, n_Words);
	else
		for (i = 0; i < n_Words; i++)
			v[i] /= n_Docs;

	// perturb the centroid k times with randomly generated vectors
	for (i = 0; i < n_Clusters; i++)
	{
		//generate a random vector
		for (j = 0; j < n_Words; j++)
			temp_v[j] = rand_gen.GetUniform() - 0.5;
		normalize_vec(temp_v, n_Words);
		temp_norm = Perturb * rand_gen.GetUniform();
		for (j = 0; j < n_Words; j++)
			temp_v[j] *= temp_norm;
		for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = v[j] + temp_v[j];
		if (Alg == SPHERICAL_K_MEANS)
			normalize_vec(Concept_Vector[i], n_Words);

		// ...
		/* for (j = 0; j < n_Words; j++)
			Concept_Vector[i][j] = docs(j, i); */
	}

/*
#ifdef DEBUG
	cout << "initial concept vectors\n";
	for (i = 0; i < n_Clusters; i++)
	{
		cout << "concept vector " << i << "\n";
		for (j = 0; j < n_Words; j++)
			cout << Concept_Vector[i][j] << " ";
		cout << "\n\n";
	}
#endif */
}

/*
void SPKMeans::refined_partition(SparseMatrixDouble *p_Docs, int j)
{
	int i, ii, iii, best_init;
	float best = 0.0, temp = 0.0;
	int* columns;
	int subsample_size;
	int index;


	VECTOR_double **cm = new (VECTOR_double *)[j];
	for (i = 0; i < j; i++)
	{
		cm[i] = new VECTOR_double[n_Clusters];
		for (ii = 0; ii < n_Clusters; ii++)
			cm[i][ii] = new float[n_Words];
	}

	int nz;
	int* row_ind, *col_ptr;
	float *val;
	SparseMatrixDouble *s, *mcm;

	for (i = 0; i < j; i++)
	{
		subsample_size = int(n_Docs * SampleSize);
		columns = new int[subsample_size];

		nz = 0;

		for ( ii = 0; ii < subsample_size; ii ++)
		{
			columns[ii] = rand_gen.GetUniformInt(n_Docs);
			//create a matrix called s
			nz += p_Docs->col_ptr(columns[ii]+1) - p_Docs->col_ptr(columns[ii]);
		}
		row_ind = new int[nz];
		val = new float[nz];
		col_ptr = new int[subsample_size + 1];

		index = 0;

		for ( ii = 0; ii < subsample_size; ii++)
		{
			col_ptr[ii] = index;
			for ( iii = p_Docs->col_ptr(columns[ii]); iii < p_Docs->col_ptr(columns[ii]+1); iii ++)
			{
				val[index] = p_Docs->val(iii);
				row_ind[index] = p_Docs->row_ind(iii);
				index ++;

			}
		}
		col_ptr[subsample_size] = nz;
		s = new SparseMatrixDouble(n_Words, subsample_size, nz, val, row_ind, col_ptr);

		k_means_mod((*s), cm[i]);
		delete s;
	}
	//create a matrix mcm according to cm

	int *col_ptr1, *row_ind1;
	float *val1;
	nz = 0;
	for ( i = 0; i < n_Clusters; i ++)
	{
		for (iii = 0; iii < j; iii++)
		for (ii = 0; ii < n_Words; ii ++)
		{
			if (cm[iii][i][ii] != 0) nz ++;
		}
	}
	val1 = new float[nz]; col_ptr1 = new int[nz + 1]; row_ind1 = new int[nz];
	index = 0;
	for ( i = 0; i <  j; i ++)
	{
		for (iii = 0; iii < n_Clusters; iii++)
		{
		col_ptr1[i*n_Clusters+iii] = index;
		for (ii = 0; ii < n_Words; ii ++)
		{
			if (cm[i][iii][ii] != 0)
			{
				val1[index] = cm[i][iii][ii];
				row_ind1[index] = ii;
				index ++;
			}
		}
		}
	}
	col_ptr1[n_Clusters * j] = nz;
	mcm = new SparseMatrixDouble(n_Words, n_Clusters * j, nz, val1, row_ind1, col_ptr1);

	SPKMeans K;
	int *cluster = new int[n_Clusters * j];

	for (i = 0; i < j; i ++)
	{
		K.Init(mcm, cluster, n_Clusters, CONCEPT_VECTORS_INIT);
//		for (ii = 0; ii < n_Clusters; ii++)
//			assert(mcm->dim(0) == cm[i][ii].dim());
		K.SetConceptVectors(cm[i]);
		K.Iterate(p_Doc);
		temp = K.GetResult(p_Doc);
		if (best < temp)
		{
			best = temp;
			best_init = i;
		}

	}

	delete mcm;
	for (i = 0; i < n_Clusters; i++)
		for (int k = 0; k < n_Words; k++)
			Concept_Vector[i][k] = cm[best_init][i][k];
	for (i = 0; i < j; i++)
		delete[] cm[i];
	delete[] cm;
}

void SPKMeans::refined_partition(DenseMatrixDouble *p_Docs, int j)
{
	int i, ii, iii, best_init;
	float best = 0.0, temp = 0.0;
	int* columns;
	int subsample_size;
	int index;


	VECTOR_double **cm = new (VECTOR_double *)[j];
	for (i = 0; i < j; i++)
	{
		cm[i] = new VECTOR_double[n_Clusters];
		for (ii = 0; ii < n_Clusters; ii++)
			cm[i][ii] = new float[n_Words];
	}

	int nz;
	int* row_ind, *col_ptr;
	float *val;
	SparseMatrixDouble *s, *mcm;

	for (i = 0; i < j; i++)
	{
		subsample_size = int(n_Docs * SampleSize);
		columns = new int[subsample_size];

		nz = 0;

		for ( ii = 0; ii < subsample_size; ii ++)
		{
			columns[ii] = rand_gen.GetUniformInt(n_Docs);
			//create a matrix called s
			//nz += p_Docs->col_ptr(columns[ii]+1) - p_Docs->col_ptr(columns[ii]);
			nz += p_Doc->GetNumCol();
		}
		row_ind = new int[nz];
		val = new float[nz];
		col_ptr = new int[subsample_size + 1];

		index = 0;

		for ( ii = 0; ii < subsample_size; ii++)
		{
			col_ptr[ii] = index;
			for ( iii = p_Docs->col_ptr(columns[ii]); iii < p_Docs->col_ptr(columns[ii]+1); iii ++)
			{
				val[index] = p_Docs->val(iii);
				row_ind[index] = p_Docs->row_ind(iii);
				index ++;

			}
		}
		col_ptr[subsample_size] = nz;
		s = new SparseMatrixDouble(n_Words, subsample_size, nz, val, row_ind, col_ptr);

		k_means_mod((*s), cm[i]);
		delete s;
	}
	//create a matrix mcm according to cm

	int *col_ptr1, *row_ind1;
	float *val1;
	nz = 0;
	for ( i = 0; i < n_Clusters; i ++)
	{
		for (iii = 0; iii < j; iii++)
		for (ii = 0; ii < n_Words; ii ++)
		{
			if (cm[iii][i][ii] != 0) nz ++;
		}
	}
	val1 = new float[nz]; col_ptr1 = new int[nz + 1]; row_ind1 = new int[nz];
	index = 0;
	for ( i = 0; i <  j; i ++)
	{
		for (iii = 0; iii < n_Clusters; iii++)
		{
		col_ptr1[i*n_Clusters+iii] = index;
		for (ii = 0; ii < n_Words; ii ++)
		{
			if (cm[i][iii][ii] != 0)
			{
				val1[index] = cm[i][iii][ii];
				row_ind1[index] = ii;
				index ++;
			}
		}
		}
	}
	col_ptr1[n_Clusters * j] = nz;
	mcm = new SparseMatrixDouble(n_Words, n_Clusters * j, nz, val1, row_ind1, col_ptr1);

	SPKMeans K;
	int *cluster = new int[n_Clusters * j];

	for (i = 0; i < j; i ++)
	{
		K.Init(mcm, cluster, n_Clusters, CONCEPT_VECTORS_INIT);
//		for (ii = 0; ii < n_Clusters; ii++)
//			assert(mcm->dim(0) == cm[i][ii].dim());
		K.SetConceptVectors(cm[i]);
		K.Iterate(p_Doc);
		temp = K.GetResult(p_Doc);
		if (best < temp)
		{
			best = temp;
			best_init = i;
		}

	}

	delete mcm;
	for (i = 0; i < n_Clusters; i++)
		for (int k = 0; k < n_Words; k++)
			Concept_Vector[i][k] = cm[best_init][i][k];
	for (i = 0; i < j; i++)
		delete[] cm[i];
	delete[] cm;
}

void Assign_Cluster(SparseMatrixDouble &docs, VECTOR_double concept_vectors[], int k,
			int cluster[])
{
	int i, j;
	VECTOR_double similarity, temp;

	similarity = new float[docs.GetNumCol()];
	temp = new float[docs.GetNumCol()];

	docs.trans_mult(concept_vectors[0], similarity);
	for (i=0; i<docs.GetNumCol(); i++)
		cluster[i] = 0;

	for (i=1; i<k; i++)
	{
		docs.trans_mult(concept_vectors[i], temp);
		for (j = 0; j < docs.GetNumCol(); j++)
		{
			if (temp[j] > similarity[j])
			{
				similarity[j] = temp[j];
				cluster[j] = i;
			}
		}
	}
}

VECTOR_double SPKMeans::farthest_data(SparseMatrixDouble &docs, VECTOR_double concept_vector[])
{
	int i;
	float min_similarity = 1.0, temp;
	int farthest = -1;
	VECTOR_double result = new float[docs.GetNumRow()];
	int *cluster = new int[docs.GetNumCol()];

	for (i = 0; i < docs.GetNumRow(); i++)
		result[i] = 0.0;
	Assign_Cluster(docs, concept_vector, n_Clusters, cluster);
	for (i = 0; i < docs.GetNumCol(); i++)
	{
		temp = dot_mult(&docs, i, concept_vector[cluster[i]]);
		if (temp < min_similarity)
		{
			min_similarity = temp;
			farthest = i;
		}
	}

	for (i = docs.col_ptr(farthest); i < docs.col_ptr(farthest+1); i++)
	{
		result[docs.row_ind(i)] = docs.val(i);
	}

	return result;
}

void SPKMeans::k_means_mod(SparseMatrixDouble &docs, VECTOR_double result_concept_vectors[])
{
	assert (docs.GetNumRow() == n_Words);

	int i, j;
	int* cluster_size, *cluster;
	cluster_size = new int[n_Clusters];
	cluster = new int[docs.GetNumCol()];
	int redo = 0; // boolean
	VECTOR_double *temp_concept_vectors = new VECTOR_double[n_Clusters];
	for (i = 0; i < n_Clusters; i++)
		temp_concept_vectors[i] = new float[docs.GetNumRow()];
	for ( i = 0; i < n_Clusters; i ++)
		for (j = 0; j < docs.GetNumRow(); j++)
			temp_concept_vectors[i][j] = Concept_Vector[i][j];
	SPKMeans K;

	do {
	redo = 0;
	K.Init(&docs, cluster, n_Clusters, CONCEPT_VECTORS_INIT);
	K.SetConceptVectors(temp_concept_vectors);
	K.Iterate();

	for ( i = 0; i < n_Clusters; i ++ ) cluster_size[i] = 0;
	for ( i = 0; i < docs.GetNumCol(); i++)
		cluster_size[cluster[i]]++;

	for ( i = 0; i < n_Clusters; i ++ )
	{
		if (cluster_size[i] == 0)
		{
			temp_concept_vectors[i] = farthest_data(docs, temp_concept_vectors);
			redo = 1;
		}
	}
	} while (redo == 1);

	for ( i = 0; i < n_Clusters; i++ )
		for (j = 0; j < docs.GetNumRow(); j++)
			result_concept_vectors[i][j] = temp_concept_vectors[i][j];
}

*/
