/*	mat_vec.cc

	Implementation of the Sparse_Mat_double class

	Copyright(c) 2000 Yuqiang Guan 
*/

#include <iostream.h>
#include <math.h>
#include "mat_vec.h"

int int_power (int x, int y)
{
  int pow =1;

  for ( int i=0; i<y; i++)
    pow *=x;

  return pow;
}

void h_normalize_mat(SparseMatrixDouble *mat, int * index, int n_docs)
{
  int i, j;
  float norm;

  for ( i = 0; i < n_docs; i++)
    {
      norm = 0.0;
      for (j = mat->col_ptr(index[i]); j < mat->col_ptr(index[i]+1); j++)
	norm += mat->val(j) * mat->val(j);

      if(norm!=0)
	{
	  norm = sqrt(norm);
	  for (j = mat->col_ptr(index[i]); j < mat->col_ptr(index[i]+1); j++)
	    mat->val(j) /= norm;
	}
    }
}

void normalize_mat(SparseMatrixDouble *mat)
{
	int i, j;
	float norm;

	for (i = 0; i < mat->GetNumCol(); i++)
	{
		norm = 0.0;
		for (j = mat->col_ptr(i); j < mat->col_ptr(i+1); j++)
		norm += mat->val(j) * mat->val(j);
//              assert(norm > 0);
		/*		if (norm == 0)
		{
			cerr << "column " << i << " has 0 norm\n";
			}*/
		if(norm!=0)
		{
		  norm = sqrt(norm);
		  for (j = mat->col_ptr(i); j < mat->col_ptr(i+1); j++)
		    mat->val(j) /= norm;
	        }
	}
}

void normalize_mat(DenseMatrixDouble *mat)
{
  int i, j;
  float norm;

  for ( i=0; i < mat->GetNumCol(); i++)
    {
      norm = 0.0;
      for (j=0; j < mat->GetNumRow(); j++)
	norm+=mat->val(j, i)*mat->val(j, i);
    
      if(norm!=0)
	{
	  norm = sqrt(norm);
	  for (j=0; j<mat->GetNumRow() ; j++)
	    mat->val(j, i) /= norm;
	}
    }
}

void h_encode_mat(SparseMatrixDouble *mat, int *index, int n_docs, int scheme)
{
  int i, j;
  int *n_doc_per_word;	// number of documents which contains a word
  float *global_term;
  int number_row;
 
  switch (scheme)
    {
    case NORM_TERM_FREQ:
      normalize_mat(mat);
      break;
    case NORM_TERM_FREQ_INV_DOC_FREQ:
      n_doc_per_word = new int[mat->GetNumRow()];
      global_term = new float[mat->GetNumRow()];

      number_row = mat->GetNumRow();

      for (i = 0; i < number_row; i++)
	n_doc_per_word[i] = 0;


      for ( i = 0; i < n_docs; i++)
	  for (j = mat->col_ptr(index[i]); j < mat->col_ptr(index[i]+1); j++)
	    n_doc_per_word[mat->row_ind(j)]++;

      /*for (i = 0; i < mat->GetNumNonzeros(); i++)
	if (mat->row_ind(i) != 0)
	n_doc_per_word[mat->row_ind(i)]++;*/

      for (i = 0; i < number_row; i++);
      {
	if (n_doc_per_word[i] == 0)
	  global_term[i] = 0;
	else
	  global_term[i]
	    = log((float)n_docs/n_doc_per_word[i]);
      }
      for ( i = 0; i < n_docs; i++)
	for (j = mat->col_ptr(index[i]); j < mat->col_ptr(index[i]+1); j++)
	  mat->val(j) *= global_term[mat->row_ind(j)];

      /*
      for (i = 0; i < mat->GetNumNonzeros(); i++)
      mat->val(i) *= mat->val(i) * global_term[mat->row_ind(i)]; */

      h_normalize_mat(mat, index, n_docs);
      delete [] n_doc_per_word;	
      delete [] global_term;
      break;
    default:
      break;
    }
}

void encode_mat(SparseMatrixDouble *mat, int scheme)
{
	int i;
	int *n_doc_per_word;	// number of documents which contains a word
	float *global_term;
	
	switch (scheme)
	{
	case NORM_TERM_FREQ:
		normalize_mat(mat);
		break;
	case NORM_TERM_FREQ_INV_DOC_FREQ:
		n_doc_per_word = new int[mat->GetNumRow()];
		global_term = new float[mat->GetNumRow()];

		for (i = 0; i < mat->GetNumRow(); i++);
			n_doc_per_word[i] = 0;
		for (i = 0; i < mat->GetNumNonzeros(); i++)
			if (mat->row_ind(i) != 0)
				n_doc_per_word[mat->row_ind(i)]++;
		for (i = 0; i < mat->GetNumRow(); i++);
		{
			if (n_doc_per_word[i] == 0)
				global_term[i] = 0;
			else
				global_term[i]
					= log((float)mat->GetNumCol()/n_doc_per_word[i]);
		}
		for (i = 0; i < mat->GetNumNonzeros(); i++)
			mat->val(i) *= mat->val(i) * global_term[mat->row_ind(i)];
		normalize_mat(mat);
		delete [] n_doc_per_word;	
		delete [] global_term;
		break;
	default:
		break;
	}
}
void encode_mat(DenseMatrixDouble *mat, int scheme)
{
	int i, j;
	int *n_doc_per_word;	// number of documents which contains a word
	float *global_term;
	
	switch (scheme)
	{
	case NORM_TERM_FREQ:
		normalize_mat(mat);
		break;
	case NORM_TERM_FREQ_INV_DOC_FREQ:
		n_doc_per_word = new int[mat->GetNumRow()];
		global_term = new float[mat->GetNumRow()];

		for (i = 0; i < mat->GetNumRow(); i++);
			n_doc_per_word[i] = 0;
		for (i = 0; i < mat->GetNumRow(); i++)
		  for(j=0; j<mat->GetNumCol(); j++)
			if (mat->val(i, j) != 0)
				n_doc_per_word[i]++;
		for (i = 0; i < mat->GetNumRow(); i++);
		{
			if (n_doc_per_word[i] == 0)
				global_term[i] = 0;
			else
				global_term[i]
					= log((float)mat->GetNumCol()/n_doc_per_word[i]);
		}
		for (i = 0; i < mat->GetNumRow(); i++)
		  for(j=0; j<mat->GetNumCol(); j++)
	 	      mat->val(i, j) *= mat->val(i, j) * global_term[i];
		normalize_mat(mat);
		delete [] n_doc_per_word;	
		delete [] global_term;
		break;
	default:
		break;
	}
	
}


// normalize the norm of vector v to 1 if it was positive
/*void normalize_vec(float *vec, int size)
{
	float norm;
	int i;

	norm = 0.0;
	for (i = 0; i < size; i++)
	norm += vec[i] * vec[i];

//      assert(norm > 0);
	if (norm == 0) return;
	norm = sqrt(norm);

	for (i = 0; i < size; i++)
	vec[i] /= norm;
}*/

void normalize_vec(float vec[], int n)
{
        float norm;
        int i;

        norm = 0.0;
        for (i = 0; i < n; i++)
        norm += vec[i] * vec[i];

//      assert(norm > 0);
        if (norm == 0) return;
        norm = sqrt(norm);

        for (i = 0; i < n; i++)
        vec[i] /= norm;
}

// compute the dot product of the col'th document vector of docs and concept_vector
float dot_mult(SparseMatrixDouble *mat, int col, float *vec)
{
	float result = 0.0;
	int j;

	for (j = mat->col_ptr(col); j < mat->col_ptr(col+1); j++)
		result += mat->val(j) * vec[mat->row_ind(j)];

	return result;
}

float dot_mult(DenseMatrixDouble *mat, int col, float *vec)
{
  float result = 0.0;
  int j;
  
  for(j=0; j<mat->GetNumRow(); j++)
      result+=mat->val(j, col)*vec[j];
  return result;
}

float dot_mult(float *v1, float *v2, int n)
{
	float result = 0.0;
	int j;

	for (j = 0; j < n; j++)
		result += v1[j] * v2[j];

	return result;
}
