#include <time.h>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include "mat_vec.h"
#include "SPKMeans.h"
#include "timerUtil.h"

#define MAX_DESC_STR_LENGTH	9
#define MAX_VER_DIGIT_LENGTH 	3
#define SPKMEANS                200
#define QRDSPK                  201

//memory for word-document matrix in CCS format
struct doc_mat
{
	char strDesc[MAX_DESC_STR_LENGTH];	// description string
	int n_row, n_col;
	int n_nz;
	int *col_ptr;
	int *row_ind;
	float *val;
};

long memory_consume=0l;

void print_help()
{
	printf("USAGE: cluster [switches] word-doc-file\n");
	printf("\t-a algorithm\n");
	printf("\t   s: spherical k-means algorithm (default)\n");
	printf("\t   e: euclidean k-means algorithm\n");
	printf("\t-i [s|p|r]\n");
	printf("\t   initialization method:\n");
 	printf("\t      s -- subsampling\n");
	printf("\t      p -- random perturbation (default)\n");
	printf("\t      r -- totally random\n");
	printf("\t      g -- read from graph partition file\n");
	printf("\t-c number-of-clusters\n");
	printf("\t-l the depth of the tree (default is 1)\n");
	printf("\t-e epsilon\n");
	printf("\t-s\n");
	printf("\t   suppress output\n");
	printf("\t-v version-number\n");
	printf("\t-n\n");
	printf("\t   no dump\n");
	printf("\t-d\n");
	printf("\t   dump the clustering process\n");
	printf("\t-p perturbation-magnitude\n");
	printf("\t   the distance between initial concept vectors\n");
	printf("\t     and the centroid will be less than this.\n");
	printf("\t-N number-of-samples\n");
	printf("\t-O the name of output matrix\n");
	printf("\t-t scaling scheme\n");
	printf("\t-K dimension reduction parameter\n");
	printf("\t-S sample-size\n");
	printf("\t   in ratio to the total number of documents\n");
	printf("\t-E encoding-scheme\n");
	printf("\t   1: normalized term frequency (default)\n");
	printf("\t   2: normalized term frequency inverse document frequency\n");
	printf("\t-o objective-function\n");
	printf("\t   1: nonweighted (default)\n");
	printf("\t   2: weighted\n");
}

void read_mat(char *fname, char *scaling, doc_mat *mat)
{
        char filename[256];
	clock_t start_clock, finish_clock;

	sprintf(filename, "%s%s", fname, "_dim");
	std::ifstream dimfile(filename);
	if(dimfile==0)
	    cout<<"Dim file "<<filename<<" can't open.\n"<<endl;
	sprintf(filename, "%s%s", fname, "_row_ccs");
	std::ifstream rowfile(filename);
	if(rowfile==0)
	    cout<<"Row file "<<filename<<" can't open.\n"<<endl;
	sprintf(filename, "%s%s", fname, "_col_ccs");
	std::ifstream colfile(filename);
	if(colfile==0)
	    cout<<"Column file "<<filename<<" can't open.\n"<<endl;
	sprintf(filename, "%s%s", fname, "_");
	sprintf(filename, "%s%s", filename, scaling);
	sprintf(filename, "%s%s", filename, "_nz");
	std::ifstream nzfile(filename);
	if(nzfile==0)
	    cout<<"Entry file "<<filename<<" can't open.\n"<<endl;

	int i;

	if(dimfile==0 || colfile==0||rowfile==0||nzfile==0)
	{
	  //cout<<"Matrix file "<< fname<<" can't open."<<endl;
	  exit(1);
	}
	//data.width(MAX_DESC_STR_LENGTH);
	//data >> mat->strDesc;
	//data >> mat->n_col >> mat->n_row >> mat->n_nz;
	start_clock = clock();
	cout<<"Reading the _dim file..."<<endl;
        dimfile >>mat->n_row>> mat->n_col>>mat->n_nz;
	dimfile.close();

	mat->col_ptr = new int[mat->n_col+1];
	mat->row_ind = new int[mat->n_nz];
	mat->val = new float[mat->n_nz];

	memory_consume+=(mat->n_col+1+mat->n_nz)*sizeof(int)+mat->n_nz*sizeof(float);


	cout<<"Reading the _col file..."<<endl;
	for (i = 0; i < mat->n_col+1; i++)
		colfile >> (mat->col_ptr)[i];
	colfile.close();

	cout<<"Reading the _row file..."<<endl;
	for (i = 0; i < mat->n_nz; i++)
		rowfile >> (mat->row_ind)[i];
	rowfile.close();

	cout<<"Reading the _nz file..."<<endl;
	for (i = 0; i < mat->n_nz; i++)
		nzfile >> (mat->val)[i];
	nzfile.close();
	finish_clock = clock();
	cout<<"Reading file time: "<<(finish_clock - start_clock)/1e6<<" seconds."<<endl;
	cout<<"Now clustering..."<<endl;
	
	//data.close();
	// validity check ...
	// use exception handling?...
}

int main(int argc, char **argv)
{
	int *cluster;
	doc_mat mat;
	
	int i, j;
	bool no_args = true;

	float coherence;
	clock_t start_clock, finish_clock, s_c, f_c;

	// parameters for the k-means algorithm
	int alg = SPHERICAL_K_MEANS;
	int K_Clusters=100, n_Clusters = 3;
	int depth =1;

	float epsilon = DEFAULT_EPSILON;
	int init_method = RANDOM_PERTURB_INIT;
	float perturb = DEFAULT_PERTURB;
	int num_sample = DEFAULT_NUM_SAMPLE;
	float sample_size = DEFAULT_SAMPLE_SIZE;
	//int encode_scheme = NORM_TERM_FREQ;
	int encode_scheme = NORM_TERM_FREQ_INV_DOC_FREQ;
	int objfun = NON_WEIGHTED_OBJ;

	// output control
	bool no_output = false;
	bool dump = false;
	bool full_dump = false;
	bool writeQT=false;
	int  scheme = SPKMEANS;
	int  top_word_num =0;

	char ver[MAX_VER_DIGIT_LENGTH+1] = "";
	char output_matrix[256]="output_matrix";
	char wordCluster[256]="wordClusterFile";
	char output_qT[256];
	char suffix[128]="tfn";
	char docs_matrix[256];
	char graph_part[256];
	
//////////////////////////////////////////////////////////////
//----------------- Read commandline arguments ---------------


	TimerUtil runtime_est;

	//start_clock = clock();
	for (argv++; *argv != NULL; argv++)
	{
		if ((*argv)[0] == '-')
		{
			switch ((*argv)[1])
			{
			case 'a':
				if ((*(++argv))[0] == 's')
					alg = SPHERICAL_K_MEANS;
				else if((*(argv))[0] == 'e')
					alg = EUCLIDEAN_K_MEANS;
				else
				  {
				    cout <<"Invalid algorithm "<<*argv<<endl;
				    print_help();
				    exit(0);
				  }
				break;
			
			case 'i':
				switch ((*(++argv))[0])
				{
				case 's':
					init_method = SAMPLE_INIT;
					break;
				case 'p':
					init_method = RANDOM_PERTURB_INIT;
					break;
				case 'r':
					init_method = RANDOM_INIT;
					break;
				case 'g':
				  init_method = GRAPH_PART;
				  strcpy(graph_part, *(++argv));
					break;
				default:
					printf("Invalid option %s\n", *argv);
					print_help();
					exit(0);
					break;
				}
				break;
			case 'c':
				n_Clusters = atoi(*(++argv));
				break;
			case 'l':
				depth = atoi(*(++argv));
				break;
			case 'K':
			        K_Clusters =atoi(*(++argv));
				scheme = QRDSPK;
				break;
			case 'T':
			  top_word_num = atoi(*(++argv));
			  break;
			case 't':
			        strcpy(suffix, *(++argv));
				break;
			case 'e':
				epsilon = atof(*(++argv));
				break;
			case 's':
				no_output = true;
				break;
			case 'v':
				strncpy(ver, *(++argv), MAX_VER_DIGIT_LENGTH);
				break;
			case 'n':
				dump = false;
				break;
			case 'd':
				full_dump = true;
				break;
			case 'O':
			        strcpy(output_matrix, *(++argv));
				strcpy(wordCluster, output_matrix);
				break;
			case 'p':
				perturb = atof(*(++argv));
				break;
			case 'N':
				num_sample = atoi(*(++argv));
				break;
			case 'S':
				sample_size = atoi(*(++argv));
				break;
			case 'E':
				encode_scheme = atoi(*(++argv));
				break;
			case 'o':
				objfun = atoi(*(++argv));
				break;
			case 'w':
			        writeQT=true;
				break;
			default:
				printf("Invalid switch %s\n", *argv);
				print_help();
				exit(0);
				break;
			}
		}
		else
		  {
		    sprintf(docs_matrix, "%s",*argv);
			read_mat(*argv,suffix,  &mat);
			no_args = false;
		}
	}

	if (no_args)
	{
		print_help();
		exit(1);
	}
//////////////////////////////////////////////////////////////

	cluster = new int[mat.n_col];
	SparseMatrixDouble R(mat.n_row, mat.n_col, mat.n_nz, mat.val, mat.row_ind, mat.col_ptr); 

	// clustering
	//start_clock = clock();
	if (alg == SPHERICAL_K_MEANS)
		normalize_mat(&R);
	if (scheme == SPKMEANS)
	  K_Clusters=n_Clusters;

	SPKMeans K(&R, cluster, K_Clusters, init_method, epsilon);
	K.SetTopWordNum(top_word_num);

	K.SetAlgorithm(alg);
	K.SetDump(!no_output & full_dump);
	if (init_method == RANDOM_PERTURB_INIT)
		K.SetPerturb(perturb);
	else if (init_method == SAMPLE_INIT)
	{
		K.SetNumSamples(num_sample);
		K.SetSampleSize(sample_size);
	}
	K.SetEncodeScheme(encode_scheme);
	K.SetObjFunc(objfun);
	

	if(init_method==GRAPH_PART)
	  K.Iterate(&R, graph_part);
	else
	  {
	    char hname[256];
	    sprintf(hname, "%s_%s", output_matrix,suffix);
	    cout<<depth<<"-level hierarchical clustering"<<endl<<endl;
	    K.Iterate(&R, depth, hname);
	  }
	//int emptyCluster=0;
	//int *cluster_size = new int[int_power(n_Clusters,depth)]; 
	//if ( scheme == SPKMEANS)
	//emptyCluster = K.h_SortCluster(cluster_size, int_power(n_Clusters,depth));
	//coherence = K.GetResult();

	/*finish_clock = clock();
	
	cout<<"\nWriting word cluster file"<<endl;
	strcat(wordCluster, "_");
	strcat(wordCluster, suffix);
	K.wordCluster(wordCluster, depth);
	*/
	
	if( scheme == QRDSPK)
	  {
	    cout << "number of iterations: " << K.GetNumIterations() << "\n";
	    cout << "objective function value: " << coherence << "\n"; 
	    cout << "computation time: " << (finish_clock - start_clock)/1e6 << " seconds\n";
	    cout << "time each iteration: " << (finish_clock - start_clock) /
			(1e6*K.GetNumIterations()) << "seconds\n";
	    /*****************************************/
	    start_clock = clock();
	    VECTOR_double *ConMat;

	    ConMat =  new VECTOR_double[mat.n_row];
	    for ( i=0; i<mat.n_row;i++)
	      ConMat[i]= new float[K_Clusters];
	    K.ReturnConVec (ConMat);
	

	    //ConMat = K.ReturnConVec();
	    DenseMatrixDouble D(mat.n_row,K_Clusters, ConMat);
		
	
	    VECTOR_double *q;
	    q = new  VECTOR_double[mat.n_row];
	    for (i =0; i< mat.n_row; i++)
	      q[i] = new float [K_Clusters];
	
	    s_c = clock();
	    D.dqrbasis(q);
	    f_c = clock();
	    cout<<"It took "<<(f_c - s_c)/ 1e6 <<" seconds to do QRD. \n";

	    VECTOR_double *qT;
	    qT = new  VECTOR_double[K_Clusters];
	    for (i =0; i< K_Clusters; i++)
	      qT[i] = new float [mat.n_row];

	    for(i=0; i<K_Clusters; i++)
	      for(j=0; j< mat.n_row; j++)
		qT[i][j]=q[j][i];

	    if(writeQT)
	      {
		strcat(output_qT, output_matrix);
		strcat(output_qT,"_qTrans");
		std::ofstream qTrans(output_qT);
		qTrans<<K_Clusters<<" "<<mat.n_row<<endl;
		for(j=0; j< mat.n_row; j++)
		  {
		    for(i=0;  i<K_Clusters; i++)
		      //for(j=0; j< mat.n_row; j++)
		      qTrans<<qT[i][j]<<"\t";
		    qTrans<<endl;
		  }
		
		qTrans.close();
		cout<<"Q\' was writen in file "<<output_qT<<endl;
	      }

	    
	    VECTOR_double *qTa;
	    qTa = new  VECTOR_double[K_Clusters];
	    for (i =0; i<K_Clusters ; i++)
	      qTa[i] = new float [mat.n_col];
	    
	    for ( i=0; i< K_Clusters; i++)
	      R.trans_mult(qT[i], qTa[i]);

	    if(writeQT)
	      {
		output_qT[0]='\0';
		strcat(output_qT, output_matrix);
		strcat(output_qT,"_qTransA");
		std::ofstream qTransA(output_qT);
		qTransA<<K_Clusters<<" "<<mat.n_col<<endl;
		for(j=0; j< mat.n_col; j++)
		  {
		    for(i=0;  i<K_Clusters; i++)
		      //for(j=0; j< mat.n_col; j++)
		      qTransA<<qTa[i][j]<<"\t";
		    qTransA<<endl;
		  }
		qTransA.close();
		cout<<"Q\'A was writen in file "<<output_qT<<endl;
	      }

	    DenseMatrixDouble B(K_Clusters, mat.n_col, qTa);
	    
	    finish_clock = clock();
	    cout<< "It took "<<(finish_clock - start_clock)/1e6 <<" seconds to do C'A \n";
	    cout<< "\n\nNow clustering "<<K_Clusters<<"X"<<mat.n_col<<" matrix"<<endl;
	    
	    /*****************************************/
	    
	    start_clock = clock();
	    //alg=EUCLIDEAN_K_MEANS;
	    if (alg == SPHERICAL_K_MEANS)
		normalize_mat(&B);
	    K.Init(&B, cluster, n_Clusters, init_method, epsilon);
	    K.SetAlgorithm(alg);
	    K.SetDump(!no_output & full_dump);

	    if (init_method == RANDOM_PERTURB_INIT)
	      K.SetPerturb(perturb);
	    else if (init_method == SAMPLE_INIT)
	      {
		K.SetNumSamples(num_sample);
		K.SetSampleSize(sample_size);
	      }
	    K.SetEncodeScheme(encode_scheme);
	    K.SetObjFunc(objfun);

	    K.Iterate(&B);

	    //int emptyCluster;
	    //int *cluster_size = new int[n_Clusters]; 
	    //emptyCluster = K.SortCluster(cluster_size);
	    coherence = K.GetResult();
       
	    finish_clock = clock();
	  }
	//////////////////////////////////////////////////////////////////
	//---------------- incremental dump of results -------------------
	// output file naming convention is:
	//	cluster				strDesc + 'cluster' + '.' + number of cluster
	//	objective function value	strDesc + 'obj' + '.' + number of cluster
	//	number of iterations		strDesc + 'iter' + '.' + number of cluster
	//	cluster sizes			strDesc + 'csize' + '.' + number of cluster
	//	time per iteration		strDesc + 'time' + '.' + number of cluster
	//
	//	If there is a version number, '.'+version number is attached to the end of filenames

	char postfix[20];
	sprintf(postfix, ".%d", n_Clusters);
	if (strcmp(ver, "") != 0)
	{
		strcat(postfix, ".");
		strcat(postfix, ver);
	}

	char browserfilepost[128];
	
	sprintf(browserfilepost, "_doctoclus_%d_%d",depth,  n_Clusters);
	strcat(output_matrix, "_");
	strcat(output_matrix, suffix);
	strcat(output_matrix, browserfilepost);
	strcat(docs_matrix, "_docs");

	std::ofstream o_m(output_matrix);
	std::ifstream docs(docs_matrix);

	
	char ch;
	o_m<<mat.n_col<<endl;
	for (i=0; i < mat.n_col; i++)
	{
	 
	  docs>>j;
	  docs>>ch;
	  //docs>>ch;
	  o_m<<cluster[i];
	  //o_m<<"\tC"<<cluster[i]<<"\t";
	  o_m<<"\t";
	  docs>>docs_matrix;
	  o_m<<docs_matrix<<endl;
	  
	}
	o_m.close();
	//finish_clock = clock();
	
	
//----------------- dump to file ------------------------
	if (dump)
	{
		char fname[MAX_DESC_STR_LENGTH+20];
		std::ofstream dump_file;

		sprintf(fname, "%scluster%s", mat.strDesc, postfix);
		dump_file.open(fname, ios::app);
		for (i = 0; i < mat.n_col-1; i++)
				dump_file << cluster[i] << "\t";
		dump_file << cluster[mat.n_col-1] << "\n"; 
		dump_file.close();
		
		sprintf(fname, "%sobj%s", mat.strDesc, postfix);
		dump_file.open(fname, ios::app);
		//dump_file << coherence << "\n"; 
		dump_file.close();
		
		sprintf(fname, "%siter%s", mat.strDesc, postfix);
		dump_file.open(fname, ios::app);
		dump_file << K.GetNumIterations() << "\n"; 
		dump_file.close();
		
		sprintf(fname, "%scsize%s", mat.strDesc, postfix);
		dump_file.open(fname, ios::app);
		/*		for (i = 0; i < n_Clusters-1; i++)
				dump_file << cluster_size[i] << "\t";
				dump_file << cluster_size[n_Clusters-1] << "\n"; */
		dump_file.close();
		
		sprintf(fname, "%stime%s", mat.strDesc, postfix);
		dump_file.open(fname, ios::app);
		dump_file << (finish_clock-start_clock)/(1e6*K.GetNumIterations()) << "\n"; 
		dump_file.close();
	}


//----------------- dump to standard output ------------------
	if (!no_output)
	{
		if (alg == SPHERICAL_K_MEANS)
			cout << "Hierarchical spherical k means algorithm\n";
		else
			cout << "euclidean k means algorithm\n";
		if( scheme == QRDSPK)
		  cout<< "First dimension reduction then clustering"<<endl;
		
		cout << "Final number of clusters: " << int_power(n_Clusters, depth) << "\n";	
		cout << "Number of documents: " << mat.n_col << "\n";
		if( scheme == QRDSPK)
		  cout << "Final number of words: " << K_Clusters << "\n";
		else
		  cout << "Number of words: " << mat.n_row<<endl;

		cout << "epsilon: " << epsilon << "\n";
		cout << "initialization method: ";

		switch (init_method)
		{
		case RANDOM_PERTURB_INIT:
			cout << "random perturbation\n";
			cout << "perturbation magnitude: " << perturb << "\n";
			break;
		case SAMPLE_INIT:
			cout << "subsampling\n";
			cout << "number of samples: " << num_sample << "\n";
			cout << "sample size: " << sample_size << "\n";
			break;
		case RANDOM_INIT:
			cout << "totally random\n";
			break;
		case GRAPH_PART:
		  cout<<"cluster graph partition result file "<<graph_part<<endl;
		  break;
		default:
			break;
		}

		cout << "encoding scheme: ";
		if (encode_scheme == NORM_TERM_FREQ)
			cout << "normalized term frequency\n";
		else
			cout << "normalized term frequency inverse document frequency\n";
		cout << "objective function: ";
		if (objfun == 1)
			cout << "nonweighted\n";
		else
			cout << "weighted\n";

		/*cout <<"emptyCluster ="<<emptyCluster<<endl;

		for (i = emptyCluster; i < int_power(n_Clusters, depth); i++)
			cout << "cluster " << i-emptyCluster << " : " << cluster_size[i] << "\n";
		*/
		//cout << "number of iterations: " << K.GetNumIterations() << "\n";
		//cout << "objective function value: " << coherence << "\n"; 

		runtime_est.setStopTime(cout, "computation time: ", K.GetNumIterations());
		//cout << "computation time: " << (float)(finish_clock - start_clock)/1.0e6 << " seconds\n";
		//cout << "time each iteration: " << (finish_clock - start_clock) /
		//	(1e6*K.GetNumIterations()) << "seconds\n";
		cout<<"Output matrix file is: "<<output_matrix<<endl;

		cout<<"Memory consumed :"<<memory_consume<<endl;
		
	}
	
	exit (0);
}

